import {async, ComponentFixture, fakeAsync, TestBed} from '@angular/core/testing';

import {FileeditorComponent} from './fileeditor.component';
import {ReactiveFormsModule} from "@angular/forms";
import {ActivatedRoute, convertToParamMap, ParamMap, Params} from "@angular/router";
import {BehaviorSubject} from "rxjs";
import {HttpClientTestingModule, HttpTestingController} from "@angular/common/http/testing";
import {ConfigurationService} from "../env/configuration.service";
import {By} from "@angular/platform-browser";

// mock the ActivatedRoute
export class MockActivatedRoute {

  private _params: Params;
  private _paramMap: ParamMap;
  private _queryParams: Params;
  private _queryParamMap: ParamMap;

  private paramsSubject = new BehaviorSubject(this._params);
  private paramMapSubject = new BehaviorSubject(this._paramMap);
  private queryParamsSubject = new BehaviorSubject(this._queryParams);
  private queryParamMapSubject = new BehaviorSubject(this._queryParamMap);

  readonly params = this.paramsSubject.asObservable();
  readonly paramMap = this.paramMapSubject.asObservable();
  readonly queryParams = this.queryParamsSubject.asObservable();
  readonly queryParamMap = this.queryParamMapSubject.asObservable();

  setParams(params: Params) {
    this._params = params;
    this._paramMap = convertToParamMap(params);
    this.paramsSubject.next(params);
    this.paramMapSubject.next(convertToParamMap(params));
  }

  setQueryParams(params: Params) {
    this._queryParams = params;
    this._queryParamMap = convertToParamMap(params);
    this.queryParamsSubject.next(params);
    this.queryParamMapSubject.next(convertToParamMap(params));
  }

  get snapshot() {
    return {
      paramMap: this._paramMap,
      queryParamMap: this._queryParamMap
    }
  }

}

export class fakeConfigService {
  static get config() {
    return {
      url: "http://localhost:4200/VlassMngr",
      rootDataDirectory: '/lustre/aoc/cluster/pipeline/vlass_test/spool',
      weblogbaseurl: 'http://webtest.aoc.nrao.edu/'
    };
  };
}

describe('FileeditorComponent', () => {
  let component: FileeditorComponent;
  let fixture: ComponentFixture<FileeditorComponent>;
  let httpMock: HttpTestingController;
  let mockActivatedRoute: MockActivatedRoute;


  beforeEach(async(() => {

    TestBed.configureTestingModule({
      declarations: [FileeditorComponent],
      providers: [
        {provide: ActivatedRoute, useClass: MockActivatedRoute},
        {provide: ConfigurationService, useValue: fakeConfigService}
      ],
      imports: [ReactiveFormsModule, HttpClientTestingModule]
    }).compileComponents();
  }));

  beforeEach(() => {
    fixture = TestBed.createComponent(FileeditorComponent);
    component = fixture.componentInstance;
    mockActivatedRoute = fixture.debugElement.injector.get(ActivatedRoute) as any;
    httpMock = TestBed.get(HttpTestingController);
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should set local params from query params and load content', fakeAsync(() => {
    mockActivatedRoute.setParams({origin: 'file_template', id: 'calibration', type: 'text', name: 'PPR.xml'});

    expect(component.origin).toBeUndefined('origin starts undefined');
    expect(component.id).toBeUndefined('id starts undefined');
    expect(component.type).toBeUndefined('type starts undefined');
    expect(component.name).toBeUndefined('name starts undefined');
    expect(component.displayedUrl).toBeUndefined('displayedUrl starts undefined');

    fixture.detectChanges(); //ngOnInit

    expect(component.origin).toEqual('file_template', 'origin set from query param');
    expect(component.id).toEqual('calibration', 'id set from query param');
    expect(component.type).toEqual('text', 'type set from query param');
    expect(component.name).toEqual('PPR.xml', 'name set from query param');
    expect(component.displayedUrl).toEqual('job/queues/calibration/workflow/files/PPR.xml', 'displayUrl updated');

    const req = httpMock.expectOne(fakeConfigService.config.url + '/services/job/queues/calibration/workflow/files/PPR.xml');
    expect(req.request.method).toBe("GET");
    req.flush('this is the file content');

    expect(component.formGroup.get('content').value).toEqual('this is the file content', 'form control set with file content');

    httpMock.verify();
  }));

  it('should send form contents to service when save button is clicked', fakeAsync(() => {
    mockActivatedRoute.setParams({origin: 'file_template', id: 'calibration', type: 'text', name: 'PPR.xml'});

    fixture.detectChanges(); //ngOnInit

    const req = httpMock.expectOne(fakeConfigService.config.url + '/services/job/queues/calibration/workflow/files/PPR.xml');
    expect(req.request.method).toBe("GET");
    req.flush('this is the file content');

    fixture.detectChanges(); // wait for the content to resolve

    let saveBtn: HTMLButtonElement = fixture.debugElement.query(By.css('.save_btn')).nativeElement;
    saveBtn.click();

    fixture.detectChanges();

    const req2 = httpMock.expectOne(fakeConfigService.config.url + '/services/job/queues/calibration/workflow/files/PPR.xml');
    expect(req2.request.method).toBe("PUT");
    req2.flush('this is the file content');

    httpMock.verify();
  }));
});
