import {async, ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {TilesComponent} from './tiles.component';
import {of} from "rxjs";
import {Tile} from "../model/tile";
import {By} from "@angular/platform-browser";
import {Component, DebugElement, Input} from "@angular/core";
import {ReactiveFormsModule} from "@angular/forms";
import {TilesService} from "../services/tiles.service";

// mock the child component
@Component({
  selector: 'app-tile',
  template: '<p>Mock App-Tile component</p>'
})
class MockAppTileComponent {
  @Input() tile: Tile;
}

describe('TilesComponent', () => {
  let component: TilesComponent;
  let fixture: ComponentFixture<TilesComponent>;
  let tilesServiceSpy: { getTilesForEpoch: jasmine.Spy }; // we'll use a spy in place of the real service

  beforeEach(async(() => {
    // create the spy to inject as a substitute for the real TileService
    tilesServiceSpy = jasmine.createSpyObj('TilesService', ['getTilesForEpoch']);

    // set up the test env with our mocks and spies
    TestBed.configureTestingModule({
      declarations: [TilesComponent, MockAppTileComponent],
      imports: [ReactiveFormsModule],
      providers: [{provide: TilesService, useValue: tilesServiceSpy}]
    })
      .compileComponents();

  }));

  beforeEach(() => {
    fixture = TestBed.createComponent(TilesComponent);
    component = fixture.componentInstance;
    tilesServiceSpy = TestBed.get(TilesService); // we want to get the 'real' instance from our env
  });

  it('should create component', () => {
    expect(component).toBeTruthy();
  });

  it('should get tiles on init', fakeAsync(() => {
    let expectedReturn = of([]);
    tilesServiceSpy.getTilesForEpoch.and.returnValue(expectedReturn);

    fixture.detectChanges(); //onInit();

    expect(tilesServiceSpy.getTilesForEpoch.calls.any()).toBeTruthy('called tile service');
    expect(component.tiles).toEqual(expectedReturn, 'fake tiles returned');
  }));


  it('should update pattern and tiles with pattern formcontrol changes', fakeAsync(() => {
    let pattern_inputDe: DebugElement = fixture.debugElement.query(By.css('#pattern'));
    let pattern_inputEl: HTMLInputElement = pattern_inputDe.nativeElement;
    let expectedReturn = of([]);
    tilesServiceSpy.getTilesForEpoch.and.returnValue(expectedReturn);

    expect(component.pattern).toEqual('', 'pattern starts empty');
    expect(component.tiles).toBeUndefined('tiles starts empty');
    fixture.detectChanges(); // ngInit
    expect(pattern_inputEl.value).toEqual('', 'pattern control is null');

    pattern_inputEl.value = 'T01';
    pattern_inputEl.dispatchEvent(new Event('input'));
    tick(500); // for the debounce
    fixture.detectChanges();
    expect(component.pattern).toEqual('T01', 'input updated pattern');
    expect(component.tiles).toEqual(expectedReturn, 'tiles was set');
  }));

  it('clicking on an epoch should update epoch and tiles', () => {

    let dropdownButtonDe: DebugElement = fixture.debugElement.query(By.css('#page-select'));
    let dropdownButtonEl: HTMLButtonElement = dropdownButtonDe.nativeElement;
    let changeEpochButtonDe: DebugElement = fixture.debugElement.query(By.css('.page-form ul li button'));
    let changeEpochButtonEl: HTMLButtonElement = changeEpochButtonDe.nativeElement;
    let expectedReturn = of([]);
    tilesServiceSpy.getTilesForEpoch.and.returnValue(expectedReturn);

    expect(component.epoch).toEqual(1, 'epoch default value');
    expect(component.tiles).toBeUndefined('tiles start empty');
    fixture.detectChanges(); // ngInit();

    expect(dropdownButtonEl.textContent).toEqual('Epoch: Epoch 1', 'UI button default text');

    changeEpochButtonEl.click();
    fixture.detectChanges();

    expect(component.epoch).toEqual(-1, 'update epoch value');
    expect(dropdownButtonEl.textContent).toEqual('Epoch: Tests', 'UI button text updated');

  });

  it('getEpochName() should translate numbers to names', () => {
    expect(component.getEpochName(-1)).toEqual('Tests');
    expect(component.getEpochName(0)).toEqual('Pilot');
    expect(component.getEpochName(1)).toEqual('Epoch 1');
  });
});
